import Store from 'store'

export const db = await Store.Open('covers', 'data')

export async function IndexPicture(picture) {
    let id = picture.format + '-' + picture.data.length

    let c = await db.Get(id)
        .catch(err => {})

    if(c == null) {
        let blob = new Blob(
            [ new Uint8Array(picture.data) ],
            { type: picture.format }
        )
        await db.Set(id, blob)
    }
    return id
}

export async function Get(id) {
    return await db.Get(id)
}

export async function Url(id) {
    return id ?
        URL.createObjectURL(await Get(id)) :
        null
}

export async function Download(release) {
    let u = new URL('https://coverartarchive.org/release/' + release.id)
    let r = await fetch(u)

    if(!r.ok)
        return

    let d = await r.json()
    let i = d.images.find(i => i.types.includes('Front')) ?? d.images[0]
    return fetch(i.thumbnails ? i.thumbnails['250'] : i.image)
        .then(r => r.blob())
}

export async function IndexRelease(release) {
    let c = await db.Get(release.id)
        .catch(err => {})

    if(c == null) {
        let b = await Download(release)
        await db.Set(release.id, b)
    }

    return release.id
}
