import * as device from 'device'
import * as cover from './cover.js'
import * as lyrics from './lyrics.js'
import * as musicbrainz from './musicbrainz.js'

export {
    cover,
    lyrics,
    musicbrainz
}

export const trackFiletypes = [
    '.wav',
    '.mp3',
    '.ogg',
    '.flac'
]

export async function State() {
    hiss.state.entries = []
    hiss.state.index = await Entry('root', device.root)
        .catch(console.error)

    for(let t of hiss.state.entries) {
        if(IsTrack(t)) {
            await Duration(t)
            if(musicbrainz.NeedsPolyfill(t)) {
                let r = await musicbrainz.Get(t)
                if(r) {
                    musicbrainz.Polyfill(t, r)
                }
            }
            await lyrics.Download(t)
        }
    }

    // for(let t of hiss.state.entries) {
    //     if(IsTrack(t)) {
    //         await lyrics.Download(t)
    //     }
    // }
}

export function IsTrack(entry) {
    return !entry.entries
}

export async function Add(hHandle, eParent) {
    let dirE = await device.Entries(hHandle)
    let out = []

    for(let [ n, h ] of dirE) {
        let e = await Entry(n, h, eParent)
        if(e != undefined) {
            out.push(e)
            // displayTrackFound(e)
        }
    }

    return out
}

export async function Entry(sName, hHandle, eParent) {
    let e = {
        handle: hHandle,
        id: hiss.state.entries.length
    }

    if(eParent) {
        e.parent = eParent
    }

    switch(hHandle.kind) {
        case 'directory':
            hiss.state.entries.push(e)
            e.tracks = []
            e.entries = await Add(hHandle, e)
            e.name = sName

            if(e.parent) {
                e.parent.tracks = e.parent.tracks.concat(e.tracks)
            }
            break
            
        case 'file':
            let ext = sName.slice(sName.lastIndexOf('.'))
            if(trackFiletypes.includes(ext)) {
                await Track(e)
                e.parent?.tracks.push(e)
                hiss.state.entries.push(e)
            } else {
                return
            }
            break
    }

    e.order = e.trackNumber ?? 0

    return e
}

export async function Track(eTrack) {
    let md = await OpenMetadata(eTrack) ?? GenerateMetadata(eTrack)

    eTrack.album = md.album
	eTrack.name = md.title
    eTrack.artist = md.artist
    eTrack.trackNumber = md.track ? parseInt(md.track) : Infinity
    eTrack.coverId = md.picture && await cover.IndexPicture(md.picture)
    if(md.lyrics?.lyrics) {
        lyrics.Wrap(eTrack, md.lyrics?.lyrics)
    }
}

export async function OpenMetadata(eTrack) {
    let d = await device.Open(eTrack.handle).then(mediaTags)
        .catch(e => console.error(e))

    return d && d.tags
}

export function GenerateMetadata(eTrack) {
    return /((?<track>\d+)\s+)?((?<artist>.+)-\s*)?\s*(?<title>.+)\s*\.\w+/
            .exec(eTrack.handle.name)
            .groups
}

const mediaTags = blob => new Promise((resolve, reject) => {
	jsmediatags.read(blob, {
		onSuccess(tags) {
			resolve(tags)
		},
		onError(err) {
			reject(err)
		}
	})
})

export async function Duration(eTrack) {
    let f = await device.Open(eTrack.handle)
    let a = new Audio(URL.createObjectURL(f))

    await new Promise((resolve, reject) => {
        a.addEventListener('canplay', () => {
            eTrack.duration = a.duration
            resolve()
        })
        a.addEventListener('error', reject)

        setTimeout(() => reject('Failed to get track duration for ', eTrack.handle.path), 4000)
    })
        .catch(console.error)
}
