import * as controller from "./controller.js"
import * as view from "./view.js"
import * as queue from "./queue.js"
import * as index from "../index/index.js"

export async function State() {
    hiss.state.browser = {
        current: hiss.state.index,
        flatView: true,
    }
}

export let Elements

export async function Init() {
    hiss.view.browser = {
        root: document.getElementById('browser'),
        menu: document.getElementById('browser-menu'),
        focus: null
    }

	Elements = new Array(hiss.state.indexSize)
	for(let entry of hiss.state.entries) {
		Elements[entry.id] = renderEntry(entry)
	}

    Render()

    {
        let a = hiss.view.actions['browser-list'] = {}
    
        a['Enter'] = {
            up(focus) {
                Cd(Entry(hiss.view.browser.focus))
            }
        }
        a[' '] = {
            down() {
                view.FocusNext()
            }
        }
    }
    {
        let a = hiss.view.actions['browser-track'] = {}
    
        a['Enter'] = a[' '] = {
            down(focus) {
                view.player.Queue(Entry(hiss.view.browser.focus))
                view.FocusNext()
            }
        }
    }
    {
        let a = hiss.view.actions['browser'] = {}
        Object.assign(a, hiss.view.actions.menu)
    
        a['Backspace'] = a['Escape'] = {
            up: Back
        }

        a['r'] = a['4'] = {
            up: ToggleFlatView
        }
    }
}

export function Entry(element) {
    return hiss.state.entries[element.dataset.id]
}

export function Render() {
    let menu = hiss.view.browser.menu

    ReorganizeList(
        menu,
        Entries()
            .sort((a, b) => a.order - b.order)
            .map(e => Elements[e.id])
    )

    view.Focus(menu.querySelector('.focus') ?? menu.children[0] ?? menu, hiss.view.browser)
    view.ScrollInListTo(hiss.view.browser.focus)
}

export function Open() {
    // May be unnecessary once each menu has its own scrolling rules
    view.ScrollInListTo(hiss.view.browser.focus)
}

export function ReorganizeList(eList, eItems) {
	while (eList.firstChild) {
		eList.removeChild(eList.lastChild)
	}
	for(let child of eItems) {
		eList.appendChild(child)
	}
}

export function renderEntry(entry) {
    let li = document.createElement('li')

    li.dataset.id = entry.id

    if(index.IsTrack(entry)) {
        populateTrackEntry(entry, li)
    } else {
        populateListEntry(entry, li)
    }

    return li
}

export function populateListEntry(entry, li) {
    let a = document.createElement('a')
    let header = document.createElement('header')

    header.innerText = entry.name
    a.appendChild(header)
    li.appendChild(a)

    li.dataset.actionSet = 'browser-list'
}

export function populateTrackEntry(eTrack, li) {
    // TODO: fix button problem on kaios
    let aside = document.createElement('aside')
    let trackNumber = document.createElement('code')

    view.populateTrackInfo(eTrack, aside)
	trackNumber.innerText = formatTrackNumber(eTrack.trackNumber)

    li.appendChild(trackNumber)
    li.appendChild(aside)

    li.dataset.actionSet = 'browser-track'
}

export function formatTrackNumber(trackNumber) {
    return trackNumber == Infinity ? '--' : trackNumber.toString().padStart(2, '0')
}

export function Entries() {
    return hiss.state.browser.flatView ?
        hiss.state.browser.current.tracks.concat(Lists()) : 
        hiss.state.browser.current.entries
}

export function Lists() {
    return hiss.state.browser.current.entries.filter(e => !index.IsTrack(e))
}

export function Cd(entry) {
    hiss.state.browser.current = entry
    Render()
}

export function Back() {
    let i = hiss.state.browser.current.id
    let p = hiss.state.browser.current.parent
    if(p) {
        Cd(p)
        view.Focus(hiss.view.browser.menu.querySelector(`[data-id="${i}"]`))
        view.ScrollInListTo(hiss.view.browser.focus)
    }
}

export function ToggleFlatView() {
    hiss.state.browser.flatView = !hiss.state.browser.flatView
    Render()
}